//
// $Id: load_shader.cpp 43 2011-01-10 05:21:09Z bambooflow $
// $Rev: 43 $
// $Author: bambooflow $
//
#include <iostream>
#include <fstream>

#include "load_shader.h"


GLuint loadShaderFromFile(GLenum type, const char *fileName)
{
  std::fstream inputFile(fileName);
  std::istreambuf_iterator<char> dataBegin(inputFile);
  std::istreambuf_iterator<char> dataEnd;
  std::string fileData(dataBegin, dataEnd);
  const char *str = fileData.c_str();

  return loadShader(type, str, fileName);
}

GLuint loadShader(GLenum type, const char *str, const char *fileName)
{
  GLuint shader;
  GLint compiled;

  shader = glCreateShader(type);
  if (shader == 0)
    return 0;

  glShaderSource(shader, 1, &str, NULL);
  glCompileShader(shader);
  glGetShaderiv(shader, GL_COMPILE_STATUS, &compiled);
  if (!compiled) {
    GLint infoLen = 0;
    glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLen);
    if (infoLen > 1 ) {
      char* infoLog = new char[sizeof(char)*infoLen];
      glGetShaderInfoLog(shader, infoLen, NULL, infoLog);
      std::cerr << "Error compiling shader: " << fileName << "\n" << infoLog << "\n";
      delete [] infoLog;
    }
    glDeleteShader(shader);
    return 0;
  }
  return shader;
}



GLuint createProgram(const char* vsName, const char* fsName)
{
  GLuint programObject;
  GLuint vertexShader;
  GLuint fragmentShader;

  vertexShader = loadShaderFromFile(GL_VERTEX_SHADER, vsName);
  fragmentShader = loadShaderFromFile(GL_FRAGMENT_SHADER, fsName);
  programObject = createProgram( vertexShader, fragmentShader );

  if (vertexShader != 0)
    glDeleteShader(vertexShader);
  if (fragmentShader != 0)
    glDeleteShader(fragmentShader);

  return programObject;
}


GLuint createProgram(GLuint vertexShader, GLuint fragmentShader)
{
  GLuint programObject;

  if ((vertexShader == 0) || (fragmentShader == 0)) return 0;

  programObject = glCreateProgram();
  if (programObject == 0) return 0;

  glAttachShader(programObject, vertexShader);
  glAttachShader(programObject, fragmentShader);

  GLint linked;
  glLinkProgram(programObject);
  glGetProgramiv(programObject, GL_LINK_STATUS, &linked);
  if (!linked) {
    GLint infoLen = 0;

    glGetProgramiv(programObject, GL_INFO_LOG_LENGTH, &infoLen);
    if (infoLen > 1) {
      char *infoLog = new char[sizeof(char)*infoLen];

      glGetProgramInfoLog(programObject, infoLen, NULL, infoLog);
      std::cerr << "Error linking program:\n" << infoLog << "\n";
      delete [] infoLog;
    }
    glDeleteProgram(programObject);
    return 0;
  }

  return programObject;
}


