#version 330

struct MaterialParam {
  vec4 ambient;
  vec4 diffuse;
  vec4 specular;
  float shininess;
};

uniform MaterialParam u_lightMaterial;
uniform MaterialParam u_material;

uniform sampler2D u_texture0;
uniform bool      u_textureEnable;

in vec3 v_P;
in vec3 v_N;
in vec3 v_L;
in vec3 v_fragmentNormal;
in vec2 v_fragmentTexcoord0;

out vec4 fragColor;

void main(void)
{
  vec3 V = normalize(-v_P);
  vec3 N = normalize(v_N); // 正規化
  vec3 L = normalize(v_L);

  vec4 ambient = u_lightMaterial.ambient * u_material.ambient;

  float nDotL = dot(N,L);
  float diffuseLight = max(nDotL, 0.0);
  vec4 diffuse = u_lightMaterial.diffuse * u_material.diffuse * diffuseLight;

//  vec3 R = reflect(-L,N); // 反射ベクトルによるスペキュラー
//  float vDotR = pow( max(dot(V,R), 0.0), u_material.shininess );
//  if (nDotL < 0.0) vDotR = 0.0;
//  vec4 specular = u_lightMaterial.specular * u_material.specular * vDotR;

  vec3 H = normalize(L+V); // ハーフベクトルによるスペキュラー
  float nDotH = pow( max(dot(N,H), 0.0) , u_material.shininess );
  if (nDotL < 0.0) nDotH = 0.0;
  vec4 specular = u_lightMaterial.specular * u_material.specular * nDotH;

  fragColor = ambient + diffuse + specular;
}

