#version 330

uniform mat4 u_modelMatrix;        // モデル・マトリックス
uniform mat4 u_viewMatrix;         // ビュー・マトリックス
uniform mat4 u_projectionMatrix;   // 射影・マトリックス
uniform vec3 u_lightPos;

in vec3 a_position;
in vec3 a_normal;
in vec2 a_texcoord0;

out vec3 v_P; // eye direction
out vec3 v_N; // normal direction
out vec3 v_L; // light direction

void main(void)
{
  mat4 modelViewMatrix = u_viewMatrix * u_modelMatrix;
  mat3 n_mat = mat3( transpose( inverse(modelViewMatrix) ) ); // normal Matrix
  vec3 lightVec = vec3(u_viewMatrix * vec4(u_lightPos.xyz, 1.0));

  v_P = vec3(modelViewMatrix * vec4(a_position,1.0));
  v_N = normalize(n_mat * a_normal);
  v_L = normalize(lightVec - v_P);

  gl_Position = u_projectionMatrix * modelViewMatrix * vec4(a_position.xyz, 1.0);
}


